<?php
/**
 *
 */
class ErrorHandler extends CErrorHandler
{
    /**
     * @var array store error pages for different error types
     * i.e
     * array(
     *       'e404'=> array(
     *           'en' => 'site/404en'
     *       )
     * )
     * Error key should starts with 'e'
     */
    public $errorPages = array();

    public function init()
    {
        parent::init();

        register_shutdown_function(array($this, 'handleFatalError'));
    }

    public function getErrorDisplayPage()
    {
        if ($this->error) {
            $code = 'e' . $this->error['code'];
            if (array_key_exists($code, $this->errorPages)) {
                if (array_key_exists(Yii::app()->language, $this->errorPages[$code])) {
                    return $this->errorPages[$code][Yii::app()->language]."?error=".$this->error['code'];
                } else {
                    $tmp = array_values($this->errorPages[$code]);
                    return array_shift($tmp);
                }
            }
        }

        return false;
    }

    public function handleFatalError()
    {
        $error = error_get_last();
        if (isset($error) && in_array($error['type'], array(E_ERROR, E_PARSE, E_COMPILE_ERROR, E_CORE_ERROR))) {
            $message = "{$error['message']}. File: {$error['file']}. Line: {$error['line']}";
            Yii::log($message, CLogger::LEVEL_ERROR, 'php');
            Yii::app()->log->processLogs(new CEvent());
        }
    }

    /**
     * Renders an error page
     * If a custom error page was defined in the configuration, it will try to render
     * this page. If the custom error page also throws an error (404, 500, ...)
     * it will fallback to the default common Gentics Portal.Node PHP error page.
     *
     * @param CController   $controller   An instance of the controller
     */
    public function renderErrorPage($controller)
    {
        $errorDisplayPage = $this->getErrorDisplayPage();

        if (!empty($errorDisplayPage)) {
            $parsedUrl = parse_url($errorDisplayPage);

            if (!empty($parsedUrl['scheme'])) {
                // If the URL has a scheme (is external), do a redirect
                $controller->redirect($errorDisplayPage);
            }

            try {
                if (!empty($parsedUrl['query'])) {
                    // parse_str: A beautifully named PHP function that converts a URL
                    // query string (first argument) into an array (second argument)
                    parse_str($parsedUrl['query'], $_GET);
                }

                // Not nice, but the only way
                $_GET['path'] = ltrim($parsedUrl['path'], '/');

                // Try to render the custom error page and exit if successful
                $controller->forward("contentSource/renderer/dynamic", true);
            } catch (Exception $e) {
                // Some error occured while trying to rendering the custom error
                // page (like 404, 500, ...)
            }
        }

        // Show the default Gentics Portal.Node PHP error page if the above failed
        $controller->render('error', $this->error + array('error_code' => $this->error['code']));
    }
}
