<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 *  Class for customizing common modules in the custom area. If module is in the custom folder and if parameter
 * $enableCustomize (of the module) is true, gpn uses module class from custom area
 */
class WebCustModule extends CWebModule
{
    /**
     * @var boolean if customizing of the module enabled
     */
    public $enableCustomize = true;
    /**
     * @var string url to the assets folder
     */
    private $_assetsUrl;
    /**
     * @var int the number of seconds in which the cached value will expire. 0 means never expire.
     */
    public $cacheTime = 60;
    /**
     * @var string the name of the main template for a mail
     */
    public $mainMailTemplate = null;
    /**
     * Initializes the module.
     * This method is called at the end of the module constructor.
     * It sets the language and import list considering settings and availability of the modules (models and components) in the custom area
     * @see preinit
     */
    public function init()
    {
        parent::init();

        $this->setLanguageFromSession();
        $this->importFiles();
    }
    /**
     * Imports the files from the import list
     */
    private function importFiles()
    {
        $importList = array();
        $moduleId = $this->getId();
        $importList = array();
        if ($this->enableCustomize) {
            $importList = $this->getCustomizedModelsImportList();
        } else {
            $importList[] = "$moduleId.models.*";
        }
        $importList[] = "$moduleId.components.*";
        $this->setImport($importList);
    }
    /**
     * Builds the import list considering settings and availability of the models and components
     * @return array import list
     */
    private function getCustomizedModelsImportList()
    {
        $moduleId = $this->getId();

        $pathModule = $this->getBasePath();
        $pathCommonModels = $pathModule . DIRECTORY_SEPARATOR . "models" . DIRECTORY_SEPARATOR;
        $pathCustomModels = $pathModule . DIRECTORY_SEPARATOR . ".." . DIRECTORY_SEPARATOR . ".." . DIRECTORY_SEPARATOR . ".."
            . DIRECTORY_SEPARATOR . "custom" . DIRECTORY_SEPARATOR . "modules" . DIRECTORY_SEPARATOR
            . $moduleId . DIRECTORY_SEPARATOR . "models" . DIRECTORY_SEPARATOR;
        $pathCustomModelsBasic = $pathCustomModels . "basic" . DIRECTORY_SEPARATOR;

        $modelAlias = array();
        $cacheId = 'custom_models_';
        if (file_exists($pathCustomModels) && is_dir($pathCustomModels)) {

            $commonModels = scandir($pathCommonModels);
            $customModels = scandir($pathCustomModels);
            //Build cache id ( CACHING )
            foreach ($customModels as $file) {

                if (is_file($pathCustomModels.$file) && $touchTime = filemtime($pathCustomModels.$file)) {
                    $cacheId .= $touchTime;
                }
            }
            $cacheId = md5($cacheId);

            if (!($modelAlias = Yii::app()->cache->get($cacheId)) || Yii::app()->request->getParam('refresh', false)) {
                function leftFilesOnly(&$dirContent){
                    foreach($dirContent as $i => &$file){
                        if(substr($file,-4) != '.php'){
                            unset($dirContent[$i]);
                        }
                    }
                }

                leftFilesOnly($commonModels);
                leftFilesOnly($customModels);

                foreach($commonModels as $model){
                    $location = 'common';
                    $modelName = explode('.',$model);
                    if (in_array($model, $customModels)) {
                        $location = 'custom';
                        if (file_exists($pathCustomModelsBasic) && is_dir($pathCustomModelsBasic)) {
                            if(substr(decoct(fileperms($pathCustomModelsBasic)), 2) != 777) {
                                @chmod($pathCustomModelsBasic, 0777);
                            }
                        } else {
                            mkdir($pathCustomModelsBasic);
                            @chmod($pathCustomModelsBasic, 0777);
                        }
                        $basicModelPath = $pathCustomModelsBasic . $modelName[0] . "Basic." . $modelName[1];
                        if (!file_exists($basicModelPath)) {
                            //Copying basic module class file to custom area
                            copy($pathCommonModels . $model, $basicModelPath);
                            //Rename the basic model class
                            $basic_model_code=file( $basicModelPath );
                            foreach ($basic_model_code as $line=>$content) {

                                if (preg_match("/^class ([a-zA-Z0-9_]+) extends CActiveRecord/",$content) !== 0 && !strpos ($content,"Basic")) {
                                    $basic_model_code[$line] = preg_replace(
                                        "/^class ([a-zA-Z0-9_]+) extends CActiveRecord/",
                                        "class \${1}Basic extends CActiveRecord",
                                        $content);
                                    break;
                                }
                            }
                            file_put_contents($basicModelPath, $basic_model_code);
                        }
                        //Adding the declaration of a basic class to import array
                        $modelAlias[] = "site.$location.modules.$moduleId.models.basic.$modelName[0]Basic";
                    }
                    $modelAlias[] = "site.$location.modules.$moduleId.models.$modelName[0]";
                }
                Yii::app()->cache->set($cacheId, $modelAlias);
            }
        }
        if (!count($modelAlias)) {
            $modelAlias[] = "$moduleId.models.*";
        }
        return $modelAlias;
    }
    /**
     * Sets app's language from session
     */
    private function setLanguageFromSession()
    {
        if (isset(Yii::app()->user) && Yii::app()->user->getState('lang') != null) {
            Yii::app()->language = Yii::app()->user->getState('lang');
        }
    }

    /**
     * Translates a message to the specified language.
     *
     * @param string $str    message
     * @param array  $params params
     * @param string $dic    dictionary
     *
     * @return string
     */
    public static function t($str = '', $params = array(), $dic = 'core')
    {
        return Yii::t(get_called_class() . "." . $dic, $str, $params);
    }

    /**
     * Returns path to assets for current module
     *
     * @return string
     */
    public function getAssetsUrl()
    {
        $moduleId = $this->getId();

        if ($this->_assetsUrl === null) {
            $assetsPath = Yii::getPathOfAlias("$moduleId.assets");
            $this->_assetsUrl = Yii::app()->getAssetManager()->publish($assetsPath, true, -1, YII_DEBUG);
        }

        return $this->_assetsUrl;
    }
}