<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Like class
 *
 */
class Like extends CActiveRecord
{

    /**
     * @var integer
     * @desc ID of current user
     */
    public $userId;

    /**
     * @desc Defines how many likers should be shown in div on mouseover
     * @var lastLikesQty
     */
    public $lastLikesQty;

    /**
     * @desc Quantity of users to be shown at the front
     */
    public $topLikesQty;


    private $_likes;
    private $_likesQty;

    /**
     *  Returns a model
     *
     * @param string $className - Class name
     *
     * @return array|CActiveRecord
     *
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * Defines a table name
     *
     * @return string
     *
     */
    public function tableName()
    {
        return '{{likes}}';
    }

    /**
     * Returns the validation rules for attributes.
     *
     * @return array
     */
    public function rules()
    {
        return array(
            array('user_id, content_id', 'required'),
            array('user_id', 'numerical', 'integerOnly' => true),
            array('content_id, createdate', 'length', 'max' => 255)
        );
    }

    /**
     * Defines model relations
     *
     * @return array
     *
     */
    public function relations()
    {
        return array(
            'profile' => array(self::BELONGS_TO, 'Profile', 'user_id'),
            'user' => array(self::BELONGS_TO, 'User', 'user_id')
        );
    }

    /**
     * Returns quantity of likes fot current content
     *
     * @return integer
     */
    public function getLikes()
    {
        if ($this->_likes === null) {
            $criteria = new CDbCriteria;
            $criteria->compare('content_id', $this->content_id);

            $criteria->order = 'createdate DESC';
            $limit = $this->lastLikesQty + $this->topLikesQty;
            if ($limit > 0) {
                $criteria->limit = $limit;
            }

            $likes = self::model()->with('profile')->findAll($criteria);
            $this->_likes = $likes;
        }

        return $this->_likes;
    }

    /**
     * Set quantity of likes fot current content
     *
     * @param array $value value
     *
     * @return void
     */
    public function setLikes($value)
    {
        $this->_likes = $value;
    }


    /**
     * Calculates and returns quantity of likes fot current content
     *
     * @return array
     */
    public function getLikesQty()
    {
        if ($this->_likesQty === null) {
            $criteria = new CDbCriteria;

            $criteria->compare('content_id', $this->content_id);

            $qty = self::model()->count($criteria);
            $this->_likesQty = $qty;
        }
        return $this->_likesQty;
    }

    /**
     * Set the quantity of likes fot current content
     *
     * @param int $value qty
     *
     * @return void
     */
    public function setLikesQty($value)
    {
        $this->_likesQty = $value;
    }
}